@@ -9,6 +9,7 @@ from dagapp.utils import (
     display_factory,
     get_from_configs,
     static_factory,
+    vector_factory,
 )
 
 
@@ -56,3 +57,19 @@ class StaticPageFunc(BasePageFunc):
         arg_types, ranges = get_from_configs(self.configs)
 
         static_factory(self.dag, nodes, funcs, values, arg_types, ranges, c1)
+
+
+class VectorizePageFunc(BasePageFunc):
+    def __call__(self):
+        if self.page_title:
+            st.markdown(f'''## **{self.page_title}**''')
+
+        c1, c2 = st.beta_columns(2)
+
+        c2.graphviz_chart(self.dag.dot_digraph())
+        funcs = get_funcs(self.dag)
+        nodes = get_nodes(self.dag)
+        # values = get_values(self.dag, funcs)
+        # arg_types, ranges = get_from_configs(self.configs)
+
+        vector_factory(self.dag, nodes, funcs, c1)
